# -*- coding: utf-8 -*-

from kaio import Options
from functools import partial

opts = Options()
get = partial(opts.get, section='Cache')


class CachesMixin(object):

    # Settings for default cache.

    @property
    def CACHE_TYPE(self):
        return get('CACHE_TYPE', 'locmem')

    @property
    def REDIS_HOST(self):
        return get('REDIS_HOST', 'localhost')

    @property
    def REDIS_PORT(self):
        return get('REDIS_PORT', 6379)

    @property
    def CACHE_REDIS_DB(self):
        return get('CACHE_REDIS_DB', 2)

    @property
    def CACHE_REDIS_PASSWORD(self):
        return get('CACHE_REDIS_PASSWORD', None)

    @property
    def CACHE_PREFIX(self):
        return get('CACHE_PREFIX', self.APP_SLUG)

    @property
    def CACHE_TIMEOUT(self):
        return get('CACHE_TIMEOUT', 3600)

    @property
    def CACHE_MAX_ENTRIES(self):
        return get('CACHE_MAX_ENTRIES', 10000)

    @property
    def DEFAULT_CACHE(self):
        if self.CACHE_TYPE == 'redis':
            CACHE = {
                'BACKEND': 'django_redis.cache.RedisCache',
                'LOCATION':  'redis://%s:%s/%s' % (self.REDIS_HOST,
                                                   self.REDIS_PORT,
                                                   self.CACHE_REDIS_DB),
                'KEY_PREFIX': self.CACHE_PREFIX,
                'TIMEOUT': self.CACHE_TIMEOUT,
                'OPTIONS': {
                    "CLIENT_CLASS": "django_redis.client.DefaultClient",
                    'MAX_ENTRIES': self.CACHE_MAX_ENTRIES,
                },
            }
            if self.CACHE_REDIS_PASSWORD is not None:
                CACHE['OPTIONS']['PASSWORD'] = self.CACHE_REDIS_PASSWORD
        elif self.CACHE_TYPE == 'locmem':
            CACHE = {
                'BACKEND': 'django.core.cache.backends.locmem.LocMemCache',
                'CACHE_PREFIX': self.CACHE_PREFIX,
                'LOCATION': 'unique-key-apsl'
            }
        else:
            CACHE = {
                'BACKEND': 'django.core.cache.backends.dummy.DummyCache',
            }

        return CACHE

    # Settings for session cache.
    # You must set SESSION_ENGINE = 'django.contrib.sessions.backends.cache'
    # (or cached_db). By default use almost same settings as default cache.

    @property
    def SESSION_CACHE_TYPE(self):
        return get('SESSION_CACHE_TYPE', self.CACHE_TYPE)

    @property
    def SESSION_REDIS_HOST(self):
        return get('SESSION_REDIS_HOST', self.REDIS_HOST)

    @property
    def SESSION_REDIS_PORT(self):
        return get('SESSION_REDIS_PORT', self.REDIS_PORT)

    @property
    def SESSION_CACHE_REDIS_DB(self):
        return get('SESSION_CACHE_REDIS_DB', 3)

    @property
    def SESSION_CACHE_REDIS_PASSWORD(self):
        return get('SESSION_CACHE_REDIS_PASSWORD', self.CACHE_REDIS_PASSWORD)

    @property
    def SESSION_CACHE_PREFIX(self):
        return get('SESSION_CACHE_PREFIX', '%s_session' % self.CACHE_PREFIX)

    @property
    def SESSION_CACHE_TIMEOUT(self):
        return get('SESSION_CACHE_TIMEOUT', None)

    @property
    def SESSION_CACHE_MAX_ENTRIES(self):
        return get('SESSION_CACHE_MAX_ENTRIES', 1000000)

    @property
    def SESSION_CACHE(self):
        # Support for Redis only
        if self.SESSION_CACHE_TYPE != 'redis' or self.SESSION_ENGINE not in (
                'django.contrib.sessions.backends.cache',
                'django.contrib.sessions.backends.cached_db'):
            return

        CACHE = {
            'BACKEND': 'django_redis.cache.RedisCache',
            'LOCATION':  'redis://%s:%s/%s' % (self.SESSION_REDIS_HOST,
                                               self.SESSION_REDIS_PORT,
                                               self.SESSION_CACHE_REDIS_DB),
            'KEY_PREFIX': self.SESSION_CACHE_PREFIX,
            'TIMEOUT': self.SESSION_CACHE_TIMEOUT,
            'OPTIONS': {
                "CLIENT_CLASS": "django_redis.client.DefaultClient",
                'MAX_ENTRIES': self.SESSION_CACHE_MAX_ENTRIES,
            },
        }

        if self.SESSION_CACHE_REDIS_PASSWORD is not None:
            CACHE['OPTIONS']['PASSWORD'] = self.SESSION_CACHE_REDIS_PASSWORD

        return CACHE

    @property
    def SESSION_CACHE_ALIAS(self):
        return get('SESSION_CACHE_ALIAS', 'session')

    # Main cache settings.

    @property
    def CACHES(self):
        caches = {'default': self.DEFAULT_CACHE}
        session_cache = self.SESSION_CACHE
        if session_cache:
            caches[self.SESSION_CACHE_ALIAS] = session_cache
        return caches



# -*- coding: utf-8 -*-

import logging
import os
from kaio import Options
from functools import partial


logger = logging.getLogger(__name__)
opts = Options()
get = partial(opts.get, section='Email')


class EmailMixin(object):
    """Settings para enviar emails"""

    # Django settings: https://docs.djangoproject.com/en/1.11/ref/settings/#email-backend

    @property
    def DEFAULT_FROM_EMAIL(self):
        return get('DEFAULT_FROM_EMAIL', 'Example <info@example.com>')

    @property
    def EMAIL_BACKEND(self):
        backend = get('EMAIL_BACKEND')
        if backend:
            return backend

        if 'django_yubin' not in self.INSTALLED_APPS:
            return 'django.core.mail.backends.smtp.EmailBackend'

        try:
            import django_yubin  # type: ignore  # noqa
        except ImportError:
            logger.warn('WARNING: django_yubin in INSTALLED_APPS but not pip installed.')
            return 'django.core.mail.backends.smtp.EmailBackend'

        try:
            from django_yubin.version import VERSION  # type: ignore  # noqa
            if VERSION[0] > 1:
                return 'django_yubin.backends.QueuedEmailBackend'
            else:
                return 'django_yubin.smtp_queue.EmailBackend'
        except Exception:
            return 'django_yubin.smtp_queue.EmailBackend'


    @property
    def EMAIL_FILE_PATH(self):
        return get('EMAIL_FILE_PATH', None)

    @property
    def EMAIL_HOST(self):
        return get('EMAIL_HOST', 'localhost')

    @property
    def EMAIL_HOST_PASSWORD(self):
        return get('EMAIL_HOST_PASSWORD', '')

    @property
    def EMAIL_HOST_USER(self):
        return get('EMAIL_HOST_USER', '')

    @property
    def EMAIL_PORT(self):
        return get('EMAIL_PORT', 25)

    @property
    def EMAIL_SUBJECT_PREFIX(self):
        return get('EMAIL_SUBJECT_PREFIX', '[Django] ')

    @property
    def EMAIL_USE_TLS(self):
        return get('EMAIL_USE_TLS', False)

    # django-yubin settings: http://django-yubin.readthedocs.org/en/latest/settings.html

    @property
    def MAILER_PAUSE_SEND(self):
        return get('MAILER_PAUSE_SEND', False)

    @property
    def MAILER_USE_BACKEND(self):
        return get('MAILER_USE_BACKEND', 'django.core.mail.backends.smtp.EmailBackend')

    @property
    def MAILER_HC_QUEUED_LIMIT_OLD(self):
        return get('MAILER_HC_QUEUED_LIMIT_OLD', 30)

    @property
    def MAILER_STORAGE_BACKEND(self):
        return get('MAILER_STORAGE_BACKEND', "django_yubin.storage_backends.DatabaseStorageBackend")

    @property
    def MAILER_STORAGE_DELETE(self):
        return get('MAILER_STORAGE_DELETE', True)

    @property
    def MAILER_FILE_STORAGE_DIR(self):
        return get('MAILER_FILE_STORAGE_DIR', "yubin")


    # deprecated, for backwards compatibility

    @property
    def MAILER_MAIL_ADMINS_PRIORITY(self):
        try:
            from django_yubin import constants
            priority = constants.PRIORITY_HIGH
        except Exception:
            priority = 1
        return get('MAILER_MAIL_ADMINS_PRIORITY', priority)

    @property
    def MAILER_MAIL_MANAGERS_PRIORITY(self):
        return get('MAILER_MAIL_MANAGERS_PRIORITY', None)

    @property
    def MAILER_EMPTY_QUEUE_SLEEP(self):
        return get('MAILER_EMPTY_QUEUE_SLEEP', 30)

    @property
    def MAILER_LOCK_WAIT_TIMEOUT(self):
        return get('MAILER_LOCK_WAIT_TIMEOUT', 0)

    @property
    def MAILER_LOCK_PATH(self):
        return get("MAILER_LOCK_PATH", os.path.join(self.APP_ROOT, "send_mail"))



# -*- coding: utf-8 -*-

from .database import DatabasesMixin    # noqa: F401
from .cache import CachesMixin    # noqa: F401
from .compress import CompressMixin    # noqa: F401
from .paths import PathsMixin    # noqa: F401
from .logs import LogsMixin    # noqa: F401
from .filerconf import FilerMixin    # noqa: F401
from .cms import CMSMixin    # noqa: F401
from .security import SecurityMixin    # noqa: F401
from .debug import DebugMixin    # noqa: F401
from .celeryconf import CeleryMixin    # noqa: F401
from .email import EmailMixin    # noqa: F401
from .sentry import SentryMixin    # noqa: F401
from .storage import StorageMixin    # noqa: F401
from .whitenoise import WhiteNoiseMixin    # noqa: F401



from kaio import Options
from functools import partial


opts = Options()
get = partial(opts.get, section='Sentry')


class SentryMixin(object):
    """Sentry configuration"""

    @property
    def SENTRY_DSN(self):
        dsn = get('SENTRY_DSN')
        if dsn:
            self.sentry_init(dsn)
            self.ignore_loggers()
        return dsn

    @property
    def SENTRY_IGNORE_LOGGERS(self):
        loggers = get('SENTRY_IGNORE_LOGGERS', 'django.security.DisallowedHost')
        return [l.strip() for l in loggers.split(',') if l]

    def sentry_init(self, dsn):
        import sentry_sdk
        sentry_sdk.init(
            dsn=dsn,
            integrations=self.integrations(),
            send_default_pii=True,  # Associate Django user.id or user's IP to errors
        )

    def integrations(self):
        from sentry_sdk.integrations.django import DjangoIntegration
        return [DjangoIntegration()]

    def ignore_loggers(self):
        from sentry_sdk.integrations.logging import ignore_logger
        for logger in self.SENTRY_IGNORE_LOGGERS:
            ignore_logger(logger)



# -*- coding: utf-8 -*-

from functools import partial

from kaio import Options


opts = Options()
get = partial(opts.get, section='Storage')


class StorageMixin(object):
    """Settings for django-storages

    Currently only supports AWS S3 with and without CloudFront.
    """

    # AWS S3 settings: http://django-storages.readthedocs.io/en/latest/backends/amazon-S3.html

    @property
    def DEFAULT_FILE_STORAGE(self):
        return get('DEFAULT_FILE_STORAGE', 'storages.backends.s3boto3.S3Boto3Storage')

    @property
    def AWS_S3_SIGNATURE_VERSION(self):
        return get('AWS_S3_SIGNATURE_VERSION', 's3v4')

    @property
    def AWS_S3_REGION_NAME(self):
        return get('AWS_S3_REGION_NAME', None)

    @property
    def AWS_S3_ENDPOINT_URL(self):
        return get('AWS_S3_ENDPOINT_URL', None)

    @property
    def AWS_S3_CUSTOM_DOMAIN(self):
        return get("AWS_S3_CUSTOM_DOMAIN", None)

    @property
    def AWS_STORAGE_BUCKET_NAME(self):
        return get('AWS_STORAGE_BUCKET_NAME', '')

    @property
    def AWS_LOCATION(self):
        return get('AWS_LOCATION', '')

    @property
    def AWS_ACCESS_KEY_ID(self):
        return get('AWS_ACCESS_KEY_ID', '')

    @property
    def AWS_SECRET_ACCESS_KEY(self):
        return get('AWS_SECRET_ACCESS_KEY', '')

    @property
    def AWS_QUERYSTRING_AUTH(self):
        return get('AWS_QUERYSTRING_AUTH', True)

    @property
    def AWS_DEFAULT_ACL(self):
        return get('AWS_DEFAULT_ACL', 'private')



from functools import partial
import logging
from kaio import Options


SUPPORTED_BROKER_TYPES = ['redis', 'rabbitmq']
DEFAULT_BROKER_TYPE = 'redis'
DEFAULT_BROKER_URL = 'django://'  # used if cannot setup redis or rabbitmq


opts = Options()
get = partial(opts.get, section='Celery')

log = logging.getLogger(__name__)


class CeleryMixin(object):
    """Celery APSL Custom mixin"""

    CELERY_DISABLE_RATE_LIMITS = True
    CELERYBEAT_SCHEDULER = 'django_celery_beat.schedulers:DatabaseScheduler'

    def _redis_available(self):
        try:
            import redis  # noqa: F401
        except ImportError:
            return False

        if not self.REDIS_PORT or not self.REDIS_HOST:
            return False

        return True

    @property
    def CELERY_DEFAULT_QUEUE(self):
        return get('CELERY_DEFAULT_QUEUE', 'celery')

    @property
    def CELERY_RESULT_BACKEND(self):
        """Redis result backend config"""

        # allow specify directly
        configured = get('CELERY_RESULT_BACKEND', None)
        if configured:
            return configured

        if not self._redis_available():
            return None

        host, port = self.REDIS_HOST, self.REDIS_PORT
        if host and port:
            return "redis://{host}:{port}/{db}".format(
                host=host,
                port=port,
                db=self.CELERY_REDIS_RESULT_DB,
            )

    @property
    def CELERY_IGNORE_RESULT(self):
        """Whether to store the task return values or not (tombstones)"""
        return get('CELERY_IGNORE_RESULT', False)

    @property
    def CELERY_RESULT_EXPIRES(self):
        return get('CELERY_RESULT_EXPIRES', 86400)  # 1 day in seconds

    @property
    def CELERY_MAX_CACHED_RESULTS(self):
        """This is the total number of results to cache before older results
        are evicted. The default is 5000."""

        return get('CELERY_MAX_CACHED_RESULTS', 5000)

    @property
    def CELERY_CACHE_BACKEND(self):
        return get('CELERY_CACHE_BACKEND', 'default')

    @property
    def CELERY_ALWAYS_EAGER(self):
        return get('CELERY_ALWAYS_EAGER', False)

    @property
    def CELERY_EAGER_PROPAGATES_EXCEPTIONS(self):
        return get('CELERY_EAGER_PROPAGATES_EXCEPTIONS', True)

    @property
    def CELERY_REDIS_RESULT_DB(self):
        try:
            return int(get('CELERY_REDIS_RESULT_DB', 0))
        except Exception:
            return 0

    @property
    def CELERY_REDIS_BROKER_DB(self):
        try:
            return int(get('CELERY_REDIS_BROKER_DB', 0))
        except ValueError:
            return 0

    @property
    def RABBITMQ_HOST(self):
        return get('RABBITMQ_HOST', 'localhost')

    @property
    def RABBITMQ_PORT(self):
        return get('RABBITMQ_PORT', 5672)

    @property
    def RABBITMQ_USER(self):
        return get('RABBITMQ_USER', 'guest')

    @property
    def RABBITMQ_PASSWD(self):
        return get('RABBITMQ_PASSWD', 'guest')

    @property
    def RABBITMQ_VHOST(self):
        return get('RABBITMQ_VHOST', '/')

    @property
    def BROKER_TYPE(self):
        """Custom setting allowing switch between rabbitmq, redis"""

        broker_type = get('BROKER_TYPE', DEFAULT_BROKER_TYPE)
        if broker_type not in SUPPORTED_BROKER_TYPES:
            log.warn("Specified BROKER_TYPE {} not supported. Backing to default {}".format(
                broker_type, DEFAULT_BROKER_TYPE))
            return DEFAULT_BROKER_TYPE
        else:
            return broker_type

    @property
    def BROKER_URL(self):
        """Sets BROKER_URL depending on redis or rabbitmq settings"""

        # also allow specify broker_url
        broker_url = get('BROKER_URL', None)
        if broker_url:
            log.info("Using BROKER_URL setting: {}".format(broker_url))
            return broker_url

        redis_available = self._redis_available()
        broker_type = self.BROKER_TYPE
        if broker_type == 'redis' and not redis_available:
            log.warn("Choosed broker type is redis, but redis not available. \
                    Check redis package, and REDIS_HOST, REDIS_PORT settings")

        if broker_type == 'redis' and redis_available:
            return 'redis://{host}:{port}/{db}'.format(
                    host=self.REDIS_HOST,
                    port=self.REDIS_PORT,
                    db=self.CELERY_REDIS_BROKER_DB)
        elif broker_type == 'rabbitmq':
            return 'amqp://{user}:{passwd}@{host}:{port}/{vhost}'.format(
                    user=self.RABBITMQ_USER,
                    passwd=self.RABBITMQ_PASSWD,
                    host=self.RABBITMQ_HOST,
                    port=self.RABBITMQ_PORT,
                    vhost=self.RABBITMQ_VHOST)
        else:
            return DEFAULT_BROKER_URL



# -*- coding: utf-8 -*-

from configurations import Configuration
from kaio import Options
from functools import partial

opts = Options()
get = partial(opts.get, section='Compress')


class CompressMixin(object):

    STATICFILES_FINDERS = list(Configuration.STATICFILES_FINDERS) + [
        "compressor.finders.CompressorFinder",
    ]

    @property
    def COMPRESS_ENABLED(self):
        return get('COMPRESS_ENABLED', False)

    @property
    def COMPRESS_CSS_HASHING_METHOD(self):
        return get('COMPRESS_CSS_HASHING_METHOD', 'content')

    @property
    def COMPRESS_DEBUG_TOGGLE(self):
        if self.DEBUG:
            return 'nocompress'
        return None

    @property
    def COMPRESS_LESSC_ENABLED(self):
        return get('COMPRESS_LESSC_ENABLED', True)

    @property
    def COMPRESS_BABEL_ENABLED(self):
        return get('COMPRESS_BABEL_ENABLED', True)

    @property
    def COMPRESS_SASS_ENABLED(self):
        return get('COMPRESS_SASS_ENABLED', True)

    @property
    def COMPRESS_LESSC_PATH(self):
        return get('COMPRESS_LESSC_PATH', 'lessc')

    @property
    def COMPRESS_BABEL_PATH(self):
        return get('COMPRESS_BABEL_PATH', 'babel')

    @property
    def COMPRESS_SASS_PATH(self):
        return get('COMPRESS_SASS_PATH', 'node-sass')

    @property
    def COMPRESS_PRECOMPILERS(self):
        precompilers = []
        if self.COMPRESS_LESSC_ENABLED:
            precompilers.append(('text/less', self.COMPRESS_LESSC_PATH + ' {infile} {outfile}'))
        if self.COMPRESS_BABEL_ENABLED:
            precompilers.append(('text/babel', self.COMPRESS_BABEL_PATH + ' {infile} -o {outfile}'))
        if self.COMPRESS_SASS_ENABLED:
            precompilers.append(('text/sass', self.COMPRESS_SASS_PATH + ' {infile} {outfile}'))
            precompilers.append(('text/scss', self.COMPRESS_SASS_PATH + ' {infile} {outfile}'))
        return precompilers

    # offline settings
    # http://django-compressor.readthedocs.org/en/latest/settings/#offline-settings

    @property
    def COMPRESS_OFFLINE(self):
        return get('COMPRESS_OFFLINE', False)

    @property
    def COMPRESS_OFFLINE_TIMEOUT(self):
        return get('COMPRESS_OFFLINE_TIMEOUT', 31536000)  # 1 year in seconds

    @property
    def COMPRESS_OFFLINE_MANIFEST(self):
        return get('COMPRESS_OFFLINE_MANIFEST', 'manifest.json')

    def COMPRESS_OUTPUT_DIR(self):
        if not self.COMPRESS_ENABLED and self.COMPRESS_LESSC_ENABLED:
            return ''
        else:
            return get('COMPRESS_OUTPUT_DIR', 'CACHE/')



#!/usr/bin/env python
# encoding utf-8

import os
import re
from setuptools import setup, find_packages
import sys


main_py = open(os.path.join('kaio', '__init__.py')).read()
metadata = dict(re.findall("__([A-Z]+)__ = '([^']+)'", main_py))
__VERSION__ = metadata['VERSION']


install_requires = [
    'clint',
    'django-configurations>=2,<3',
]
if sys.version_info[0] < 3:
    install_requires.append('configparser')


with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()


setup(
    name='django-kaio',
    version=__VERSION__,
    author='APSL',
    author_email='engineering@apsl.net',
    url='https://github.com/APSL/django-kaio',
    packages=find_packages(),
    license='BSD',
    description="Class based settings for Django projects that can be read from multiple sources",
    long_description=README,
    install_requires=install_requires,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'Framework :: Django :: 1.9',
        'Framework :: Django :: 1.10',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Framework :: Django :: 2.1',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Framework :: Django :: 3.1',
        'Framework :: Django :: 3.2',
    ],
    include_package_data=True,
    zip_safe=False,
)
